/*  -*- c -*-
   $RCSfile: canvastriangle.c,v $
   $Revision: 1.6 $
   $Author: reida $
   $Date: 2005/06/23 14:49:24 $  */

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modifed
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

/* Custom canvas item, draws an equilateral triangle on the canvas
   whose edge size is given in pixels and whose orientation is given
   in radians.  */

#include "canvastriangle.h"
#include <libart_lgpl/art_vpath.h>
#include <stdio.h>

/* #define N_DOT_POINTS 36 */

static GtkCanvasREClass *parent_class;


/* Shape path drawer.  Gets the center, edge size, and angle. */
void private_shape_gen(ArtVpath *vpath, double x0, double y0, 
		       double size, double angle)
{
  int i; 
  double cos(), sin(), sqrt();
  double sq3,th3;

  sq3 = size/sqrt(3.0);
  th3 = (2.0*M_PI)/3.0;

  for(i=0;i<=3;i++) { 
    vpath[i].code = (i==0 ? ART_MOVETO : ART_LINETO);
    vpath[i].x = x0 + sq3*sin(-i*th3-angle);
    vpath[i].y = y0 - sq3*cos(-i*th3-angle);
  }
  vpath[4].code = ART_END;
  vpath[4].x = 0.0;
  vpath[4].y = 0.0;
}

static void gtk_canvas_triangle_update(GtkCanvasItem *item,
				       double affine[6],
				       ArtSVP *clip_path, gint flags)
{
  GtkCanvasRE *re;
  GtkCanvasTriangle *triangle;
  ArtVpath vpath[5];  /* Start, draw first side, draw second side, draw-back-to-start, end marker. */
  ArtVpath *vpath2;
  double w_center_x, w_center_y, w_radius;
  extern GtkCanvasREClass *parent_class;
  /* Utility functions originally defined in gtk-canvas-rect-ellipse.c */
  void set_gc_foreground(GdkGC*, gulong);
  void set_stipple(GdkGC*, GdkBitmap**, GdkBitmap*, int);
  void get_bounds(GtkCanvasTriangle*, int*, int*, int*, int*);

  re = GTK_CANVAS_RE(item);
  triangle = GTK_CANVAS_TRIANGLE(item);
  
  /* Chain parent-class updates.  Fails at compile time becuase
     the class has no "update" member... */
  /* if (parent_class->update) 
     (*parent_class->update)(item, affine, clip_path, flags); */
  
  if(!item->canvas->aa) {
    set_gc_foreground(re->fill_gc, re->fill_pixel); 
    set_stipple(re->fill_gc, &re->fill_stipple, re->fill_stipple, TRUE);
  }
    
  if (item->canvas->aa) {
    /* Want to draw the shape in the canvas coordinates.  Transform
       the origin, then draw. */
    vpath[0].code = ART_MOVETO;
    vpath[0].x = triangle->x;
    vpath[0].y = triangle->y;
    
    vpath[1].code = ART_END;
    vpath[1].x = 0;
    vpath[1].y = 0;
    
    /* Transform the center of the triangle to world coordinates. */
    vpath2 = art_vpath_affine_transform(vpath, affine);
    w_center_x = vpath2[0].x;
    w_center_y = vpath2[0].y;
    w_radius = (double)triangle->size;
    art_free(vpath2);

    /* Overwrite vpath with the world-space path for the triangle. */
    private_shape_gen(vpath, w_center_x, w_center_y, 
		      w_radius, triangle->angle);
    
    gtk_canvas_item_update_svp_clip(item, &re->fill_svp, 
				    art_svp_from_vpath(vpath), clip_path);
  }
  else {
    int x0, x1, y0, y1;
    get_bounds(triangle, &x0, &y0, &x1, &y1);
    gtk_canvas_update_bbox(item, x0, y0, x1, y1);
  }
}


static void gtk_canvas_triangle_render(GtkCanvasItem *item,
				       GtkCanvasBuf *buffer)
{
  GtkCanvasRE *re;

  re = GTK_CANVAS_RE(item);

  if(re->fill_svp != 0) 
    gtk_canvas_render_svp(buffer, re->fill_svp, re->fill_color);
}


static void gtk_canvas_triangle_draw(GtkCanvasItem *item,
				     GdkDrawable *drawable,
				     int x, int y, int width, int height)
{
  int i;
  double i2w[6], w2c[6], i2c[6];
  double sqrt(), sin(), cos();
  double sq3, th3;
  ArtPoint i_center, c_center;
  
  GtkCanvasRE *re;
  GtkCanvasTriangle *triangle;

  GdkPoint path[3];

  re = GTK_CANVAS_RE(item);
  triangle = GTK_CANVAS_TRIANGLE(item);

  /* Retrieve affine transformations. */
  gtk_canvas_item_i2w_affine(item, i2w);
  gtk_canvas_w2c_affine(item->canvas, w2c);
  
  /* Build the item-to-canvas transformation. */
  art_affine_multiply(i2c, i2w, w2c);

  /* Use the item-to-canvas transformation to find the
     coordinates of the triangle center in canvas pixel coordinates. */
  i_center.x = triangle->x;
  i_center.y = triangle->y;
  art_affine_point(&c_center, &i_center, i2c);
  
  if (re->fill_stipple) 
    gtk_canvas_set_stipple_origin(item->canvas, re->fill_gc);
  
  sq3 = ((double)triangle->size)/sqrt(3.0);
  th3 = (2.0*M_PI)/3.0;

  for(i=0;i<3;i++) {
    path[i].x = c_center.x + sq3*sin(-i*th3-triangle->angle) - x;
    path[i].y = c_center.y - sq3*cos(-i*th3-triangle->angle) - y;
  }

  gdk_draw_polygon(drawable, re->fill_gc, TRUE, 
		   path, 3);
  
  
}


/* Event-processing helper function -- reveals whether or not the
   point in question is inside or outside the triangle. Points
   are passed in in item and canvas coords. */
static double gtk_canvas_triangle_point(GtkCanvasItem *item,
					double x, double y, int cx, int cy,
					GtkCanvasItem **actual_item)
{
  GtkCanvasTriangle *triangle;
  double sq3, th3;
  double cos(), sin(), sqrt();
  ArtPoint dx, vertex[3], interior[3];  /* Vertices, interior direction. */
  int i;

  triangle = GTK_CANVAS_TRIANGLE(item);
  *actual_item = item;
  
  /* Want radial coordinate in item space. */
  sq3 = ((double)triangle->size)/(sqrt(3.0)*(item->canvas->pixels_per_unit));
  th3 = (2.0*M_PI)/3.0;
  
  for(i=0;i<3;i++) {
    vertex[i].x =  sq3*sin(-i*th3-triangle->angle);
    vertex[i].y =  -sq3*cos(-i*th3-triangle->angle);
  }

  /* Triangle is traversed clockwise, so interior directions are
     (y2-y1, x1-x2). */
  for(i=0;i<3;i++) {
    interior[i].x = vertex[(i+1)%3].y-vertex[i].y;
    interior[i].y = vertex[i].x-vertex[(i+1)%3].x;
  }

  dx.x = x-triangle->x;
  dx.y = y-triangle->y;

  /* Take the dot products -- if any is negative, we are outside.  */ 
  for(i=0;i<3;i++) {
    if ( ((dx.x-vertex[i].x)*interior[i].x + 
	  (dx.y-vertex[i].y)*interior[i].y) < 0.0 )
      return dx.x*dx.x+dx.y*dx.y;
  }
  return 0.0;

}


static void get_bounds (GtkCanvasTriangle *triangle, int *px1, int *py1, 
			int *px2, int *py2)
{
  GtkCanvasItem *item;
  double x, y, w_size;
  int cx, cy;

  item = GTK_CANVAS_ITEM(triangle);
  
  x = triangle->x;
  y = triangle->y;

  gtk_canvas_item_i2w (item, &x, &y);

  w_size = ((double)triangle->size)/(item->canvas->pixels_per_unit);
  
  
  gtk_canvas_w2c (item->canvas, x, y, &cx, &cy);
  
  /* Bounding box is in canvas coordinates. */
  *px1 = cx-(triangle->size)-1; /* Leftmost */
  *py1 = cy-(triangle->size)-1; /* Graphically top, smallest y */
  *px2 = cx+(triangle->size)+1;
  *py2 = cy+(triangle->size)+1;
}




/******************************************************************/
/* Generic Gtk-object methods follow.                             */
/******************************************************************/

/* Local arguments.  Others are available in parent class. */
enum {
  ARG_0,  /* Dummy, arg_ids must be greater than zero. */
  ARG_X, 
  ARG_Y, 
  ARG_SIZE,
  ARG_ANGLE
};


static void gtk_canvas_triangle_class_init(GtkCanvasTriangleClass*);
static void gtk_canvas_triangle_init(GtkCanvasTriangle*);
GtkType gtk_canvas_triangle_get_type();
static void gtk_canvas_triangle_set_arg(GtkObject*, GtkArg*, guint);
static void gtk_canvas_triangle_get_arg(GtkObject*, GtkArg*, guint);



/* Set up the function pointers, register signals and 
   register object arguments. */
static void gtk_canvas_triangle_class_init(GtkCanvasTriangleClass *klass) {
  GtkObjectClass *object_class;
  GtkCanvasItemClass *item_class;

  extern GtkCanvasREClass *parent_class;
  
  object_class = (GtkObjectClass*) klass;
  item_class = (GtkCanvasItemClass*) klass;

  parent_class = gtk_type_class (GTK_CANVAS_TYPE_CANVAS_RE);

  gtk_object_add_arg_type("GtkCanvasTriangle::x",
			  GTK_TYPE_DOUBLE,
			  GTK_ARG_READWRITE,
			  ARG_X);
  gtk_object_add_arg_type("GtkCanvasTriangle::y",
			  GTK_TYPE_DOUBLE,
			  GTK_ARG_READWRITE,
			  ARG_Y);
  gtk_object_add_arg_type("GtkCanvasTriangle::size",
			  GTK_TYPE_UINT,
			  GTK_ARG_READWRITE,
			  ARG_SIZE);
  
  gtk_object_add_arg_type("GtkCanvasTriangle::angle",
			  GTK_TYPE_DOUBLE,
			  GTK_ARG_READWRITE,
			  ARG_ANGLE);

  object_class->set_arg = gtk_canvas_triangle_set_arg;
  object_class->get_arg = gtk_canvas_triangle_get_arg;

  item_class->update = gtk_canvas_triangle_update;
  item_class->render = gtk_canvas_triangle_render;
  item_class->draw   = gtk_canvas_triangle_draw;
  item_class->point  = gtk_canvas_triangle_point;
}

/* Set up the data members with reasonable defaults. */
static void gtk_canvas_triangle_init(GtkCanvasTriangle *triangle) {
  triangle->x = 0.0;
  triangle->y = 0.0;
  triangle->size = 1;
  triangle->angle = 0.0;
}

GtkType gtk_canvas_triangle_get_type(void) 
{
  static GtkType triangle_type = 0;

  if (triangle_type==0) {
    static const GtkTypeInfo triangle_info = 
      { "GtkCanvasTriangle",
	sizeof(GtkCanvasTriangle),
	sizeof(GtkCanvasTriangleClass),
	(GtkClassInitFunc) gtk_canvas_triangle_class_init,
	(GtkObjectInitFunc) gtk_canvas_triangle_init,
	0, 0, /* Legacy reserved stuff */
	(GtkClassInitFunc) 0 /* gtk_canvas_triangle_base_init */
      };
    triangle_type = gtk_type_unique(GTK_CANVAS_TYPE_CANVAS_RE, &triangle_info);
  }
  return triangle_type;
}

static void gtk_canvas_triangle_set_arg(GtkObject *object,
				   GtkArg *arg, 
				   guint arg_id) 
{
  GtkCanvasTriangle *triangle;
  GtkCanvasItem *item;

  triangle = GTK_CANVAS_TRIANGLE(object);
  item = GTK_CANVAS_ITEM(object);

  switch(arg_id) {
  case ARG_X:
    triangle->x = GTK_VALUE_DOUBLE(*arg);
    gtk_canvas_item_request_update(item);
    break;
  case ARG_Y:
    triangle->y = GTK_VALUE_DOUBLE(*arg);
    gtk_canvas_item_request_update(item);
    break;
  case ARG_SIZE:
    triangle->size = GTK_VALUE_UINT(*arg);
    gtk_canvas_item_request_update(item);
    break;
  case ARG_ANGLE:
    triangle->angle = GTK_VALUE_DOUBLE(*arg);
    gtk_canvas_item_request_update(item);
    break;
  default:
    break;
  }
}


static void gtk_canvas_triangle_get_arg(GtkObject *object,
					GtkArg *arg,
					guint arg_id) 
{
  GtkCanvasTriangle *triangle;
  triangle = GTK_CANVAS_TRIANGLE(object);

  switch(arg_id) {
  case ARG_X:
    GTK_VALUE_DOUBLE(*arg) = triangle->x;
    break;
  case ARG_Y:
    GTK_VALUE_DOUBLE(*arg) = triangle->y;
    break;
  case ARG_SIZE:
    GTK_VALUE_UINT(*arg) = triangle->size;
    break;
  case ARG_ANGLE:
    GTK_VALUE_DOUBLE(*arg) = triangle->angle;
    break;
  default:
    break;
  }
}


/* Utility functions cut-and-pasted out of gtk-canvas-rect-ellipse.c,
   because they're static and can't be linked-to. */

/* Sets the stipple pattern for the specified gc */
static void set_stipple (GdkGC *gc, GdkBitmap **internal_stipple, 
			 GdkBitmap *stipple, int reconfigure)
{
  if (*internal_stipple && !reconfigure)
    gdk_bitmap_unref (*internal_stipple);
  
  *internal_stipple = stipple;
  if (stipple && !reconfigure)
    gdk_bitmap_ref (stipple);
  
  if (gc) {
    if (stipple) {
      gdk_gc_set_stipple (gc, stipple);
      gdk_gc_set_fill (gc, GDK_STIPPLED);
    } else
      gdk_gc_set_fill (gc, GDK_SOLID);
  }
}

/* Convenience function to set a GC's foreground color to the
   specified pixel value */
static void set_gc_foreground (GdkGC *gc, gulong pixel)
{
  GdkColor c;
  
  if (!gc)
    return;
  
  c.pixel = pixel;
  gdk_gc_set_foreground (gc, &c);
}

