# -*- python -*-
# $RCSfile: pixelinfoGUI.py,v $
# $Revision: 1.23 $
# $Author: reida $
# $Date: 2005/06/03 18:08:30 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.SWIG.common import pixelgroup
from oof2.SWIG.common import switchboard
from oof2.common import debug
from oof2.common import primitives
from oof2.common.IO import pixelinfo
from oof2.common.IO.GUI import mousehandler
from oof2.common.IO.GUI import toolboxGUI
from gtk import *

#####################

# PixelInfoPlugIn classes need the following things:
#   * ordering, at the class level
#   * nrows, at the class level
#   * A constructor with arguments (GtkTable, row) which inserts nrows
#     of widgets into the table starting at the given row.
#   * A function update(toolbox, iPoint) that updates the widgets
#   * A function close() that's called when the toolbox is closed.
#   * A function nonsense() that's called when the mouse click isn't sensible.

# Here's a nearly useless baseclass:
class PixelInfoPlugIn:
    def __init__(self, toolbox):
        self.toolbox = toolbox
    def close(self):
        del self.toolbox
    def update(self, point):
        pass
    def nonsense(self):
        pass
    

plugInClasses = []

def pluginsorter(a,b):
    if a.ordering < b.ordering: return -1
    if a.ordering > b.ordering: return 1
    return 0

def registerPlugInClass(plugin):
    plugInClasses.append(plugin)
    plugInClasses.sort(pluginsorter)
    switchboard.notify('new pixelinfo plugin')

######################

class PixelInfoToolboxGUI(toolboxGUI.GfxToolbox, mousehandler.MouseHandler):
    def __init__(self, pixelinfotoolbox):
        debug.mainthreadTest()
        toolboxGUI.GfxToolbox.__init__(self, "Pixel Info", pixelinfotoolbox)
        mainbox = GtkVBox()
        self.gtk.add(mainbox)

        self.table = GtkTable(rows=2, cols=2)
        mainbox.add(self.table, expand=0, fill=0)
        
        label = GtkLabel('x=')
        label.set_alignment(1.0, 0.5)
        self.table.attach(label, 0,1, 0,1, xpadding=5, xoptions=FILL)
        self.xtext = GtkEntry()
        self.table.attach(self.xtext, 1,2, 0,1,
                          xpadding=5, xoptions=EXPAND|FILL)
        label = GtkLabel('y=')
        label.set_alignment(1.0, 0.5)
        self.table.attach(label, 0,1, 1,2, xpadding=5, xoptions=FILL)
        self.ytext = GtkEntry()
        self.table.attach(self.ytext, 1,2, 1,2,
                          xpadding=5, xoptions=EXPAND|FILL)


        self.xtsignal = self.xtext.connect('changed', self.pointChanged)
        self.ytsignal = self.ytext.connect('changed', self.pointChanged)

        box = GtkHButtonBox()
        self.updatebutton = GtkButton('Update Information')
        box.add(self.updatebutton)
        self.updatebutton.connect('clicked', self.updateButtonCB)
        self.clearbutton = GtkButton('Clear')
        box.add(self.clearbutton)
        self.clearbutton.connect('clicked', self.clearButtonCB)
        self.table.attach(box, 0,2, 2,3,
                          xpadding=5, xoptions=EXPAND|FILL, yoptions=0)

        self.updatebutton.set_sensitive(0)
        self.clearbutton.set_sensitive(0)
        self.buildGUI()
        
        self.sbcallbacks = [
            switchboard.requestCallback(pixelinfotoolbox, self.update),
            switchboard.requestCallback('new pixelinfo plugin', self.buildGUI),
            switchboard.requestCallback((self.gfxwindow(),
                                         'layers changed'), self.update)
            ]

    def buildGUI(self):
        debug.mainthreadTest()
        self.table.resize(rows=4, cols=2)
        row = 4
        self.plugins = []
        for pluginclass in plugInClasses:
            self.table.resize(row + 1 + pluginclass.nrows, 2)
            self.table.attach(GtkHSeparator(), 0,2, row,row+1,
                         ypadding=3, xoptions=EXPAND|FILL)
            self.plugins.append(pluginclass(self, self.table, row+1))
            row += pluginclass.nrows+1

    def activate(self):
        toolboxGUI.GfxToolbox.activate(self)
        self.gfxwindow().setMouseHandler(self)

    def deactivate(self):
        toolboxGUI.GfxToolbox.deactivate(self)
        self.gfxwindow().removeMouseHandler()

    def close(self):
        map(switchboard.removeCallback, self.sbcallbacks)
        for plugin in self.plugins:
            plugin.close()
        self.plugins = []

    def acceptEvent(self, eventtype):
        return eventtype == 'up'

    def up(self, x, y, shift, ctrl):
        msOrImage = self.gfxwindow().topmost('Microstructure', 'Image')
        if msOrImage:
            where = msOrImage.pixelFromPoint(primitives.Point(x,y))
            self.toolbox.menu.Query(x=where.x, y=where.y)
        else:
            for plugin in self.plugins:
                plugin.nonsense();
            

    def currentPixel(self):
        return self.toolbox.currentPixel()
    
    def update(self):
        where = self.currentPixel()
        
        if where is not None:
            self.clearbutton.set_sensitive(1)
        else:
            self.xtext.signal_handler_block(self.xtsignal)
            self.ytext.signal_handler_block(self.ytsignal)
            self.xtext.set_text('')
            self.ytext.set_text('')
            self.xtext.signal_handler_unblock(self.xtsignal)
            self.ytext.signal_handler_unblock(self.ytsignal)
            self.clearbutton.set_sensitive(0)
            
        msOrImage = self.gfxwindow().topmost('Microstructure', 'Image')
        if where is not None and msOrImage is not None:
            self.xtext.set_text(`where.x`)
            self.ytext.set_text(`where.y`)
            size = msOrImage.sizeInPixels()
            if 0 <= where.x < size.x and 0 <= where.y < size.y:
                for plugin in self.plugins:
                    plugin.update(where)
            else:
                for plugin in self.plugins:
                    plugin.nonsense();
            
            self.updatebutton.set_sensitive(0)

    def updateButtonCB(self, button):
        self.updatebutton.set_sensitive(0)
        x = int(self.xtext.get_text())
        y = int(self.ytext.get_text())
        msOrImage = self.gfxwindow().topmost('Microstructure', 'Image')
        size = msOrImage.sizeInPixels()
        if x < 0 or x >= size.x or y < 0 or y >= size.y: # illegal size
            self.update()               # restore original values
            return
        self.toolbox.menu.Query(x=x, y=y)

    def clearButtonCB(self, button):
        self.toolbox.menu.Clear()

    def pointChanged(self, *args):
        debug.mainthreadTest()
        self.updatebutton.set_sensitive(1)

    def findMicrostructure(self):
##         who = self.toolbox.gfxwindow().topwho('Microstructure', 'Image',
##                                               'Skeleton', 'Mesh')
##         if who is not None:
##             return who.getMicrostructure()
##         return None
        return self.toolbox.findMicrostructure()

def _makeGUI(self):
    return PixelInfoToolboxGUI(self)

pixelinfo.PixelInfoToolbox.makeGUI = _makeGUI

####################################

class MicrostructurePlugIn(PixelInfoPlugIn):
    ordering = 2
    nrows = 2
    def __init__(self, toolbox, table, row):
        debug.mainthreadTest()
        PixelInfoPlugIn.__init__(self, toolbox)
        label = GtkLabel('microstructure=')
        label.set_alignment(1.0, 0.5)
        table.attach(label, 0,1, row,row+1, xpadding=5, xoptions=FILL)
        self.microtext = GtkEntry()
        self.microtext.set_editable(0)
        table.attach(self.microtext, 1,2, row,row+1,
                     xpadding=5, xoptions=EXPAND|FILL)

        label = GtkLabel('pixel groups=')
        label.set_alignment(1.0, 0.5)
        table.attach(label, 0,1, row+1,row+2, xpadding=5, xoptions=FILL)
        self.grouplist = GtkList()
        scroll = GtkScrolledWindow()
        scroll.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        scroll.add_with_viewport(self.grouplist)
        table.attach(scroll, 1,2, row+1,row+2, xpadding=5, xoptions=EXPAND|FILL)

        self.sbcallbacks = [
            switchboard.requestCallback('changed pixel group', self.grpchanged),
            switchboard.requestCallback('destroy pixel group', self.grpdestroy),
            switchboard.requestCallback('renamed pixel group', self.grprenamed)
            ]

    def close(self):
        map(switchboard.removeCallback, self.sbcallbacks)
        PixelInfoPlugIn.close(self)
        
    def update(self, where):
        debug.mainthreadTest()
        microstructure = self.toolbox.findMicrostructure()
        self.grouplist.clear_items(0, -1)
        if microstructure and where is not None:
            self.microtext.set_text(microstructure.name())
            for name in pixelgroup.pixelGroupNames(microstructure, where):
                self.grouplist.add(GtkListItem(name))
            self.grouplist.show_all()
        else:
            self.microtext.set_text('(No microstructure)')

    def nonsense(self):
        debug.mainthreadTest()
        self.grouplist.clear_items(0, -1)
        self.microtext.set_text('???')
    def grpchanged(self, group, ms_name):
        microstructure = self.toolbox.findMicrostructure()
        if microstructure and microstructure.name() == ms_name:
            if group.name() in microstructure.groupNames():
                self.update(self.toolbox.currentPixel())
    def grpdestroy(self, group, ms_name):
        microstructure = self.toolbox.findMicrostructure()
        if microstructure and microstructure.name() == ms_name:
            self.update(self.toolbox.currentPixel())                
    def grprenamed(self, group, oldname, newname):
        microstructure = self.toolbox.findMicrostructure()
        if microstructure:
            if newname in microstructure.groupNames():
                self.update(self.toolbox.currentPixel())

registerPlugInClass(MicrostructurePlugIn)
