# -*- python -*-
# $RCSfile: pixelselection.py,v $
# $Revision: 1.15 $
# $Author: langer $
# $Date: 2005/06/08 18:28:10 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 


# There is one set of selected pixels for each Microstructure.  It's
# maintained as a list of iPoints and as a BitmapOverlay for quick
# drawing.

# PixelSelections have to know their Microstructures at the C++ level
# so that they can find their active areas.

from oof2.SWIG.common import switchboard
from oof2.SWIG.common import cpixelselection
from oof2.common import color
from oof2.common import debug
from oof2.common.IO import bitoverlaydisplay
from oof2.common.IO import display
from oof2.common.IO import whoville
from oof2.common.IO import topwho       # required for '<top microstructure>'
import types

class PixelSelection(cpixelselection.CPixelSelection):
    def __repr__(self):
        return "PixelSelection(%s)" % len(self)
    def getMicrostructure(self):
        cms = self.getCMicrostructure()
        # Find which Microstructure corresponds to this CMicrostructure.
        return common.microstructure.microStructures[cms.name()]

#####################################

class PixelSelectionContext(whoville.WhoDoUndo):
    def getMicrostructure(self):
        return self.getObject().getMicrostructure()
    def start(self):
        newselection = self.getObject().clone()
        self.pushModification(newselection)

    # These have an embedded active area check.
    def select(self, selectioncourier):
        self.getObject().select(selectioncourier)
    def unselect(self, selectioncourier):
        self.getObject().unselect(selectioncourier)
    def toggle(self, selectioncourier):
        self.getObject().toggle(selectioncourier)
    def selectSelected(self, selectioncourier):
        self.getObject().selectSelected(selectioncourier)
    def clearAndSelect(self, selectioncourier):
        self.getObject().clear()
        self.getObject().select(selectioncourier)
    def selectFromGroup(self, group):
        # Used when copying the pixel selection from one
        # Microstructure to another.  Does *not* check the active
        # area.
        debug.fmsg(group.__class__.__name__)
        self.getObject().setFromGroup(group)
    def clear(self):
        self.getObject().clear()
    def invert(self):
        self.getObject().invert()
    def getSelection(self):
        return self.getObject().members()
    def getSelectionAsGroup(self):
        return self.getObject().getPixelGroup()
    def undo(self):
        self.undoModification()
    def redo(self):
        self.redoModification()
    def clearable(self):
        return self.size() != 0
    def size(self):
        return len(self.getObject())
    def getBitmap(self):
        return self.getObject().getBitmap()

##################
    
pixelselectionWhoClass = whoville.WhoDoUndoClass(
    'Pixel Selection',
    instanceClass=PixelSelectionContext,
    ordering=999,
    secret=1,
    proxyClasses=['<top microstructure>'])

###################

display.predefineLayer(
    'Pixel Selection',
    '<top microstructure>',
    bitoverlaydisplay.bitmapOverlay(color=color.RGBColor(1.0, 0.22, 0.09),
                                    opacity=0.6))
