// -*- C++ -*-
// $RCSfile: petsc_preconditioner.h,v $
// $Revision: 1.12 $
// $Author: reida $
// $Date: 2003/09/23 18:19:56 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#ifdef HAVE_PETSC

#include <oofconfig.h>

#ifndef PETSCPRECONDITIONER_H
#define PETSCPRECONDITIONER_H
#include "petscvec.h"
#include "petscmat.h"
#include "petscsles.h"
#include "engine/preconditioner.h"
#include <string>
#include "engine/csrmatrix.h"

class PETScPreconditioner;
class PETScILU_Preconditioner;
class PETScIC_Preconditioner;
class PETScNULL_Preconditioner;

class PETScPreconditioner : public Preconditioner {
public:
  PETScPreconditioner();
  virtual VECTOR_D solve(const VECTOR_D&) const {};
  virtual VECTOR_D trans_solve(const VECTOR_D&) const {};
  virtual ~PETScPreconditioner() {};
  virtual void set(PC &precond) const = 0;
  
};

class PETScILU_Preconditioner : public PETScPreconditioner {
public:
  PETScILU_Preconditioner();
  virtual void set(PC &precond) const;  
};


class PETScIC_Preconditioner : public PETScPreconditioner {
public:
  PETScIC_Preconditioner();
  virtual void set(PC &precond) const {PCSetType(precond,PCICC); };
};


class PETScNULL_Preconditioner : public PETScPreconditioner {
public:
  PETScNULL_Preconditioner();
  virtual void set(PC &precond) const {PCSetType(precond,PCNONE); };
};



class PETScPreconditionerWrap : public PreconditionerWrap {
public:
  static const std::string modulename_;
  virtual const std::string &modulename() const { return modulename_; }
};

class PETScILUPreconditioner: public PETScPreconditionerWrap { //wrapper for core class
public:
  PETScILUPreconditioner() {};
  ~PETScILUPreconditioner() {};
  virtual Preconditioner *  create_preconditioner(const CSRmatrix &A, 
						  const std::vector<int> &rg, 
						  const std::vector<int> &cg) {
    return new PETScILU_Preconditioner();};
  static const std::string classname_;
  virtual const std::string &classname() const { return classname_; }
};


class PETScICPreconditioner : public PETScPreconditionerWrap { //wrapper for core class
public:
  PETScICPreconditioner() {};
  ~PETScICPreconditioner() {};
  virtual Preconditioner * create_preconditioner(const CSRmatrix &A, 
						 const std::vector<int> &rg, 
						 const std::vector<int> &cg) { 
    return new PETScIC_Preconditioner();};
  static const std::string classname_;
  virtual const std::string &classname() const { return classname_; }
};

class PETScNULLPreconditioner : public PETScPreconditionerWrap { //wrapper for core class
public:
  PETScNULLPreconditioner() {};
  ~PETScNULLPreconditioner() {};
  virtual Preconditioner *  create_preconditioner(const CSRmatrix &A, 
						  const std::vector<int> &rg, 
						  const std::vector<int> &cg) {
    return new PETScNULL_Preconditioner();};
  static const std::string classname_;
  virtual const std::string &classname() const { return classname_; }

};
#endif // PETSCPRECONDITIONER_H


#endif //HAVE_PETSC


