# -*- python -*-
# $RCSfile: steppingscheme_.py,v $
# $Revision: 1.28 $
# $Author: redwing $
# $Date: 2005/06/01 14:02:33 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.common import cregisteredclass
from oof2.common import debug
from oof2.common import registeredclass
from oof2.common.IO import parameter
from oof2.common.IO import reporter
from oof2.common.IO import mainmenu
from oof2.common.IO import oofmenu
from oof2.common.IO import xmlmenudump
from oof2.SWIG.common import devel
from oof2.SWIG.common.IO import progressbar


## TODO WAY WAY LATER: Make stepping schemes aware of the equations that
## they should (and can) solve.

cregisteredclass.registerCClass(TimeDependentPtr)

TimeDependentPtr.tip = "Solution methods for time dependent equations"


def __TimeDependentIterate(self, meshcontext):
    pbar = progressbar.getProgress()
    if (meshcontext.getObject().current_time() >= self.get_tf()) or pbar.query_stop() > 0:
        return None
    return 1

TimeDependentPtr.iterate = __TimeDependentIterate

def __TimeDependentUpdate(self, meshcontext):
    self.update_mesh(meshcontext.getObject())
    reporter.report("t = %g"%meshcontext.getObject().current_time())

TimeDependentPtr.update = __TimeDependentUpdate

cregisteredclass.Registration(
    'QuasiStatic',
    TimeDependentPtr, ## base class
    QuasiStatic, ## subclass
    ordering=100,
    params = [
    parameter.FloatParameter("dt", 1,
                        tip="time step"),
    parameter.FloatParameter("tf", 10,
                             tip="target time")
    ],
    tip = "For meshes whose properties are a function of time, but their associated partial differential equations have NO time derivatives (in steady state)"
    )

class FiniteDifferenceBase (registeredclass.RegisteredClass):
    registry = []
    def get_value(self):
        return self.value
    def get_name(self):
        return self.name
    
class Euler (FiniteDifferenceBase):
    name = "Euler"
    def __init__(self):
        pass

registeredclass.Registration(
    'Euler',
    FiniteDifferenceBase,
    Euler,
    ordering = 100,
    params = [],
    tip = 'Euler method'
    )

class CrankNicolson (FiniteDifferenceBase): ## nothing to do with Jack
    name = "CrankNicolson"
    def __init__(self):
        pass

registeredclass.Registration(
    'CrankNicolson', ## see comment above
    FiniteDifferenceBase,
    CrankNicolson,
    ordering = 200,
    params = [],
    tip = 'Crank Nicolson method'
    )

class Galerkin (FiniteDifferenceBase):
    name = "Galerkin"
    def __init__(self):
        pass
        
registeredclass.Registration(
    'Galerkin',
    FiniteDifferenceBase,
    Galerkin,
    ordering = 300,
    params = [],
    tip = 'Galerkin method'
    )

class BackwardDifferences (FiniteDifferenceBase):
    name = "BackwardDifferences"
    def __init__(self):
        pass
        

        
registeredclass.Registration(
    'BackwardDifferences',
    FiniteDifferenceBase,
    BackwardDifferences,
    ordering = 400,
    params = [],
    tip = 'Backward Differences method'
    )

class Liniger (FiniteDifferenceBase):
    name = "Liniger"
    def __init__(self):
        pass
        
registeredclass.Registration(
    'Liniger',
    FiniteDifferenceBase,
    Liniger,
    ordering = 500,
    params = [],
    tip = 'Very stable (and convenient) method for resolving sharp initial conditions. Minimizes Gibbs instabilities.'
    )

class Custom (FiniteDifferenceBase):
    name = "Custom"
    def __init__(self, value):
        self.value = value
       
registeredclass.Registration(
    'Custom',
    FiniteDifferenceBase,
    Custom,
    ordering = 600,
    params = [
    parameter.FloatParameter("value",
                             0.878,
                             tip = "pick a number, any number..."
                             )
    ],
    tip = 'Custom method'
    )

__OLD_Kinetic_init = Kinetic.__init__

def NEW_Kinetic_init(self, dt, tf, finitedifference):
    ## The current implementation of this init function
    ## is because each steppingscheme object may have
    ## different (and/or multiple) values associated to
    ## each type of finite difference method. For example,
    ## an Euler kinetic steppingscheme has one parameter (theta),
    ## but a dynamic one will have at least two (theta1, theta2). 
    if isinstance(finitedifference, Euler):
        val = 0.0
    elif isinstance(finitedifference, CrankNicolson): 
        val = 0.5
    elif isinstance(finitedifference, Galerkin):
        val = 2.0/3.0
    elif isinstance(finitedifference, BackwardDifferences):
        val = 1.0
    elif isinstance(finitedifference, Liniger):
        val = 0.878
    elif isinstance(finitedifference, Custom):
        val = finitedifference.get_value()
    else:
        raise Exception("Not a valid finite difference method!!!")
    __OLD_Kinetic_init(self, dt, tf, val)

Kinetic.__init__ = NEW_Kinetic_init

cregisteredclass.Registration(
    "Kinetic",
    TimeDependentPtr,
    Kinetic,
    ordering=200,
    params=[
    parameter.FloatParameter("dt", 0.1,
                             tip="time step"),
    parameter.FloatParameter("tf", 0.1,
                             tip="target time"),
    parameter.ActiveListRegisteredParameter("finitedifference",
                                           FiniteDifferenceBase,
                                           activelist = [Euler, CrankNicolson, Galerkin, BackwardDifferences, Liniger, Custom],
                                           tip="type of finite differences scheme"),
    ],
    tip="time step method for solving diffusion equations ONLY"
    )


## THIS SECTION IS FOR DEBUGGING
##
## def _sum_test(menuitem):
##     sum_matrices()

##mainmenu.OOF.Help.addItem(oofmenu.OOFMenuItem(
##    'SumTest',
##    callback=_sum_test,
##    threadable = oofmenu.UNTHREADABLE,
##    params=[],
##    help="sum two sparse matrices",
##    discussion=xmlmenudump.loadFile('')
##    ))
