# -*- python -*-
# $RCSfile: oofimageIPC.py,v $
# $Revision: 1.9 $
# $Author: langer $
# $Date: 2005/05/13 18:44:21 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 


from oof2.common import debug
from oof2.common import primitives
from oof2.common.IO import mainmenu
from oof2.common.IO import microstructureIO
from oof2.common.IO import oofmenu
from oof2.common.IO import parallelmainmenu
from oof2.common.IO import parameter
from oof2.common.IO import socket2me
from oof2.common.IO import whoville
from oof2.SWIG.common import cmicrostructure
from oof2.SWIG.common import mpitools


_rank = mpitools.rank()
_size = mpitools.size()
## OOF.LoadData.IPC.Microstructure
ipcoofimagemenu = parallelmainmenu.ipcmenu.addItem(oofmenu.OOFMenuItem('Image', secret=1, no_log=1))


## OOF.LoadData.IPC.Microstructure.Initialize
## Transfers the microstructure to the back-end

def _parallel_init_img(menuitem, image):
    ## image is the image name
    debug.fmsg()
    global _rank
    ## front-end work
    if _rank == 0:
        ## resolve image name and transfer the image to the back-end
        from oof2.SWIG.image import oofimage
        ## debug.fmsg("getting image")
        img_obj = oofimage.getImage(image)
        ## debug.fmsg("sending image")
        oofimage.send_image(img_obj, _rank)
        ## debug.fmsg("finished sending")
    ## back-end part of the work
    else:
        debug.fmsg()
        ## back-end reconstructs the image and adds it to the imagecontext
        from oof2.SWIG.image import oofimage
        ##  debug.fmsg("receiving image")
        img_obj_copy = oofimage.receive_image(0)
        ## debug.fmsg("adding image")
        oofimage.imageContexts.add(image, img_obj_copy)
        ## debug.fmsg("finished receiving")

        



ipcoofimagemenu.addItem(oofmenu.OOFMenuItem('Initialize', callback = _parallel_init_img, secret=1, no_log=1, threadable = oofmenu.PARALLEL_THREADABLE, params=[parameter.StringParameter('image')] ))


def _push_img(menuitem, image):
    ## image is the image name
    global _rank
    ## front-end work
    if _rank == 0:
        ## resolve image name and transfer the image to the back-end
        from oof2.SWIG.image import oofimage
        img_obj = oofimage.getImage(image) 
        oofimage.send_image(img_obj, _rank) ## send the image
        
    ## back-end part of the work
    else:
        debug.fmsg()
        ## back-end reconstructs the image and pushes it into the imagecontext
        from oof2.SWIG.image import oofimage
        img_obj_copy = oofimage.receive_image(0)
        imagecontext = oofimage.imageContexts[image]
        imagecontext.reserve()
        try:
            imagecontext.begin_writing()
            try:
                oofimage.pushModification(image, img_obj_copy)
            finally:
                imagecontext.end_writing()
        finally:
            imagecontext.cancel_reservation()
        
ipcoofimagemenu.addItem(oofmenu.OOFMenuItem('Push', callback = _push_img, secret=1, no_log=1, threadable = oofmenu.PARALLEL_THREADABLE, params=[parameter.StringParameter('image')] ))

def _undo_img(menuitem, image):
    ## image is the image name
    global _rank
    ## front-end work
    if _rank == 0:
        pass
        
    ## back-end part of the work
    else:
        debug.fmsg()
        ## back-end reconstructs the image and pushes it into the imagecontext
        from oof2.SWIG.image import oofimage
        imagewho = oofimage.imageContexts[image]
        try:
            imagewho.begin_writing()
            oofimage.undoModification(image)
        finally:
            imagewho.end_writing()
ipcoofimagemenu.addItem(oofmenu.OOFMenuItem('Undo', callback = _undo_img, secret=1, no_log=1, threadable = oofmenu.PARALLEL_THREADABLE, params=[parameter.StringParameter('image')] ))


def _redo_img(menuitem, image):
    ## image is the image name
    global _rank
    ## front-end work
    if _rank == 0:
        pass
        
    ## back-end part of the work
    else:
        debug.fmsg()
        ## back-end reconstructs the image and pushes it into the imagecontext
        from oof2.SWIG.image import oofimage
        imagewho = oofimage.imageContexts[image]
        try:
            imagewho.begin_writing()
            oofimage.redoModification(image)
        finally:
            imagewho.end_writing()
ipcoofimagemenu.addItem(oofmenu.OOFMenuItem('Redo', callback = _redo_img, secret=1, no_log=1, threadable = oofmenu.PARALLEL_THREADABLE, params=[parameter.StringParameter('image')] ))


def _create_pixel_groups(menuitem, image):
    debug.fmsg()
    ## microstructure is the name of the microstructure
    global _rank
    ## front-end work
    if _rank == 0:
        pass            
    ## back-end part of the work
    else:
        from oof2.image.IO import imagemenu
        mainmenu.OOF.Image.AutoGroup(image = image)
        

        

ipcoofimagemenu.addItem(oofmenu.OOFMenuItem(
    'AutoGroup',
    callback = _create_pixel_groups,
    secret=1,
    no_log=1,
    threadable = oofmenu.PARALLEL_THREADABLE,
    params=[whoville.WhoParameter('image', whoville.getClass('Image'))]
    ))
