# -*- python -*-
# $RCSfile: adaptive.py,v $
# $Revision: 1.3 $
# $Author: haan $
# $Date: 2005/05/17 20:21:24 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.tutorials import tutorial
TutoringItem = tutorial.TutoringItem
TutorialClass = tutorial.TutorialClass

TutorialClass(
    subject = "Adaptive Mesh Refinement",
    ordering = 6,
    lessons = [
    
    TutoringItem(
    subject="Introduction",
    comments=

    """OOF2 provides a rudimentary adaptive mesh refinement tool
    using BOLD(a Posteriori) error estimation scheme,
    BOLD(Superconvergent Patch Recovery) originally designed by
    BOLD(Zienkiewicz) and BOLD(Zhu) -- more discussion on the subject
    can be found in the OOF2 maunal.

    In this tutorial, the adaptive mesh refinement
    will be briefly demonstrated.
    """),

    TutoringItem(
    subject="Loading a Skeleton",
    comments=
    
    """Open a graphics window, if none has been opened yet, with
    the BOLD(Graphics/New) command in the BOLD(Windows) menu.

    Download the file BOLD(el_shape.skeleton) from
    http://www.ctcms.nist.gov/oof/examples, or locate it within the
    share/oof2/examples directory in your OOF2 installation.

    A data file can be loaded from BOLD(File) menu in the main OOF2
    window (BOLD(File -> Load -> Data)).
    Select the example file (BOLD(el_shape.skeleton)) in the file selector,
    and click BOLD(OK).
    """,
    signal = ("new who", "Skeleton")
    ),
    
    TutoringItem(
    subject="L-shaped Domain",
    comments=

    """The Skeleton is not rectangular. OOF2 cannot generate
    a non-rectangular Skeleton natively but such Skeletons (Meshes) can be
    loaded into OOF2. Note that the base Microstructure, however, is
    rectangular.

    The loaded Skeleton contains most of vital ingredients such as
    boundaries, properties, and materials.

    Move on to the next slide to create a Mesh from this Skeleton.
    """ ),

    TutoringItem(
    subject="Finite Element Mesh",
    comments=
    """Open the BOLD(FE Mesh) page.

    Click the BOLD(New) button to get a dialog box for creating a new
    Mesh.  Here, you must specify which types of mesh elements to use
    for triangular (BOLD(element3)) and quadrilateral (BOLD(element4))
    skeleton elements and their orders.

    Use these values, BOLD(map_order) = 2, BOLD(fun_order) = 2,
    BOLD(element3) = T6_6, BOLD(element4) = Q8_8 to create
    a quadratic Mesh.

    Click BOLD(OK).
    """,
    signal = ("new who", "Mesh")
    ),

    TutoringItem(
    subject="Field",
    comments=

    """The boundary value problem that we're going to solve is
    the same one that we did in the BOLD(Unusual Mesh) tutorial.
    The BOLD(top) and BOLD(left) side is fixed and the BOLD(right) side
    will be pulled down, thus creating a region of stress concentration
    at the corner.
    
    Proceed to the BOLD(Fields) page.
    
    Check all BOLD(three) boxes for the BOLD(Displacement) field.
    """,
    signal = "field activated"
    ),

    TutoringItem(
    subject="Activating Equations",
    comments=

    """Open the BOLD(Equations) page.

    We're solving BOLD(Force_Balance) equation, so check the corresponding box.
    """,
    signal = "equation activated"
    ),

    TutoringItem(
    subject="Boundary Conditions",
    comments="""Go to the BOLD(Boundary Conditions) page.

    The boundary conditions we're going to apply are:

    BOLD(1.) u_x = 0 on the BOLD(left) side
    
    BOLD(2.) u_y = 0 on the BOLD(left) side

    BOLD(3.) u_x = 0 on the BOLD(top) side

    BOLD(4.) u_y = 0 on the BOLD(top) side

    BOLD(5.) u_y = -2 on the BOLD(right) side"""
    ),

    TutoringItem(
    subject="Boundary Condition 1",
    comments=
    """Click the BOLD(New...) button from the BOLD(Condition) pane to bring
    up a boundary condition builder.  The pull-down menu at the top of
    the dialog box allows you to choose the type of boundary
    condition.  Set it to BOLD(Dirichlet) boundary conditions, which
    gives Fields fixed values at the boundaries.
    
    Since only one Field is defined and only one Equation is active,
    the BOLD(field) and BOLD(equation) menus have only one choice
    each.  Leave them set to BOLD(Displacement) and
    BOLD(Force_Balance).
 
    The first B.C. deals with displacement in the BOLD(x)-direction,
    so select BOLD(x) for both BOLD(Displacement) and
    BOLD(Force_Balance).

    The BOLD(profile) is the functional form of the Field along the
    boundary. BOLD(profile) to BOLD(Constant Profile) with BOLD(value) = 0.

    Choose the BOLD(boundary) to which this condition should
    be applied (BOLD(left)) and click BOLD(OK).
    """,
    signal = "boundary conditions changed"
    ),

    TutoringItem(
    subject="Boundary Condition 2-5",
    comments=
    """Create rest of Boundary Conditions as we did for the first one.
    You can always go back to previous slides to check some numbers and
    stuff by clicking on the BOLD(Back) button. BUT, make sure
    to create all the BCs before you move on to the next slide.
    """,
    signal = "boundary conditions changed"
    ),

    TutoringItem(
    subject="Solution",
    comments=
    
    """Open the BOLD(Solver) page and just click BOLD(Solve).
    Deformed Mesh will be displayed in the graphics window.

    Due to the shape of the doamin, it is obvious that stresses are
    highly concentrated in the region surrounding the corner.
    It is safe to assume that errors in this region would be higher than
    other regions.

    Move on to the next slide to start the process for adaptive mesh
    refinement.
    """,
    signal = "mesh changed"
    ),

    TutoringItem(
    subject="Adaptive Mesh Refinement",
    comments=
    
    """Go back to the BOLD(FEMesh) page.

    Select BOLD(Adapive Mesh Refinement).
    As of now, we have only one error estimator, BOLD(Z-Z Estimator).
    Select BOLD(L2 Error Norm) for error estimating BOLD(method).
    Select BOLD(stress), which is the only entity,
    for the BOLD(flux) parameter.
    Set BOLD(threshold) to be BOLD(10).

    For each element, an L2 error norm will be computed
    from stresses from the finite element solution and their
    recovered counterparts, which act as exact stresses.
    If the relative error exceeds 10 percent, the element will be refined.

    The next two parameters, BOLD(criterion) and BOLD(degree) take care of
    actual refinement.

    Sometimes, refinement could create badly-shaped elements. These elements
    can be removed by turning on the BOLD(rationalize) option.

    The displacement field can be transferred to the refined mesh by
    turning on the BOLD(init_field) option. This, however, is just a
    projection of the previous solution onto the refined mesh.

    Leave these four option as they are for now and click BOLD(OK).
    """,
    signal = "mesh changed"
    ),

    TutoringItem(
    subject="Refined Mesh",
    comments=
    
    """As expected, elements surrounding the corner have been refined.

    Now, go to the BOLD(Solver) page.
    BOLD(Solve) the problem again with the refined mesh.
    """,
    signal = "mesh changed"
    ),

    TutoringItem(
    subject="Refine Again",
    comments=
    
    """
    Go back to the BOLD(FEMesh) page and refine the mesh again
    (just click BOLD(OK)).
    
    The corner has been refined more. For a better view, use
    BOLD(ctrl)+BOLD(.) or BOLD(Settings)->BOLD(Zoom)->BOLD(In) from
    the graphics window.

    This process (BOLD(Refine) + BOLD(Solve)) can be repeated, until
    you're satisfied.

    Thanks for trying out the tutorial.
    """,
    signal = "mesh changed"
    )
    
    ])
